"""
First see if you can id the object; if so done.
Second, if not, if there is an object in front, move toward it.
Third, else if there is an object on the left or right, turn toward it.
Fourth, else move forward.  Only stop with objectIDed.
"""

from planBaseClass import PlanBaseClass
from timerClass import TimerClass

class PlanClass (PlanBaseClass):
    NUMBER_GOALS = 1
    goalIDObject = 0

    NUMBER_MODULES = 6
    moduleIDObject = 0
    moduleForwardToObject = 1
    moduleTurnToObject = 2
    moduleMoveForwardNoInfo = 3
    moduleTurnRight = 4
    moduleTurnLeft = 5

    NUMBER_FACTS = 5
    objectIsDogFact = 0
    objectIsCatFact = 1
    objectInFrontFact = 2
    objectOnRightFact = 3
    objectOnLeftFact = 4

    NUMBER_ACTIONS = 4
    actionRight = 0
    actionLeft = 1
    actionForward = 2
    actionBackward = 3

    def __init__(self, simName,sim,neal,spinnVersion,fsa):
        PlanBaseClass.__init__(self,simName,sim,neal,spinnVersion,fsa)
        self.simName = simName
        self.sim = sim
        self.neal = neal
        self.spinnVersion = spinnVersion
        self.fsa = fsa
        self.initializeBasePlanner()

    def makeTimers(self):
        self.iDObjTimer = TimerClass(self.simName,self.sim,self.neal,
                                            self.spinnVersion,self.fsa,10)
        self.forwardTimer = TimerClass(self.simName,self.sim,self.neal,
                                            self.spinnVersion,self.fsa,5)
        self.turnTimer = TimerClass(self.simName,self.sim,self.neal,
                                            self.spinnVersion,self.fsa,5)

    def IDObj(self,modCells,factCells):
        #If you can id the obj, we're done
        self.factStopsGoal(self.objectIsDogFact,self.goalIDObject)
        self.factStopsModule(self.objectIsDogFact,self.moduleIDObject)
        self.factStopsGoal(self.objectIsCatFact,self.goalIDObject)
        self.factStopsModule(self.objectIsCatFact,self.moduleIDObject)
        #timer slides to next module unless the goal stops
        self.iDObjTimer.makeOneStateStopTimerSynapses()
        self.iDObjTimer.stateStartsTimer(modCells,self.moduleIDObject)
        #timer is stopped by either id fact
        self.iDObjTimer.stateStopsTimer(factCells,self.objectIsDogFact)
        self.iDObjTimer.stateStopsTimer(factCells,self.objectIsCatFact)
        #the objectIsFacts make areset
        self.factStartsReset(self.objectIsDogFact)
        self.factStartsReset(self.objectIsCatFact)

    def moveForwardOnFact(self,modCells):
        #else if obj in front, move forward
        self.iDObjTimer.timerStartsState(modCells,self.moduleForwardToObject)
        self.moduleStopsModule(self.moduleForwardToObject,self.moduleIDObject)
        self.moduleHalfStartsAction(self.moduleForwardToObject,
                                    self.actionForward)
        self.factHalfStartsAction(self.objectInFrontFact,self.actionForward)
        self.actionStopsModule(self.actionForward,self.moduleForwardToObject)
        #timer slides to turn module unless a foward fact
        self.forwardTimer.makeOneStateStopTimerSynapses()
        self.forwardTimer.stateStartsTimer(modCells,self.moduleForwardToObject)
        #action stops timer
        self.actionStopsTimer(self.actionForward,self.forwardTimer)

    def turnToObjectOnFact(self,modCells):
        self.forwardTimer.timerStartsState(modCells,self.moduleTurnToObject)
        self.moduleStopsModule(self.moduleTurnToObject,
                               self.moduleForwardToObject)
        #need to prevent the moduleIDObject
        self.moduleStopsModule(self.moduleTurnToObject,
                               self.moduleIDObject)
        #hook up module and facts
        self.moduleAndFactStartModuleWithAction(self.moduleTurnToObject,
            self.objectOnLeftFact,self.moduleTurnLeft,self.actionLeft)
        self.actionStopsModule(self.actionLeft,self.moduleTurnToObject)
        self.moduleAndFactStartModuleWithAction(self.moduleTurnToObject,
            self.objectOnRightFact,self.moduleTurnRight,self.actionRight)
        self.actionStopsModule(self.actionRight,self.moduleTurnToObject)
        #timer slides to forward default module unless a l/r fact
        self.turnTimer.makeOneStateStopTimerSynapses()
        self.turnTimer.stateStartsTimer(modCells,self.moduleTurnToObject)
        #action stops timer
        self.actionStopsTimer(self.actionRight,self.turnTimer)
        self.actionStopsTimer(self.actionLeft,self.turnTimer)

    def connectIDObjectPlanner(self):
        self.makeTimers()
        self.allActionsCauseFullReset()
        modCells = self.getModuleCells()
        factCells = self.getFactCells()

        self.goalStartsModule(self.goalIDObject,self.moduleIDObject)

        #If you can id the obj, we're done
        self.IDObj(modCells,factCells)

        #else if obj in front, move forward
        self.moveForwardOnFact(modCells)

        #else if obj on side, turn toward it
        self.turnToObjectOnFact(modCells)

        #else move forward
        self.turnTimer.timerStartsState(modCells,self.moduleMoveForwardNoInfo)
        self.moduleStopsModule(self.moduleMoveForwardNoInfo,
                                self.moduleTurnToObject)
        self.moduleStartsAction(self.moduleMoveForwardNoInfo,self.actionForward)
        self.actionStopsModule(self.actionForward,self.moduleMoveForwardNoInfo)
        #prevent first module

    def printResults(self):
        PlanBaseClass.printResults(self)
        self.iDObjTimer.printResults("results/iDObjTimer")
        self.forwardTimer.printResults("results/forwardTimer")
        self.turnTimer.printResults("results/turnTimer")

